@@ -1,6 +1,6 @@
 {- Standard git remotes.
  -
- - Copyright 2011-2021 Joey Hess <id@joeyh.name>
+ - Copyright 2011-2023 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU AGPL version 3 or higher.
  -}
@@ -11,7 +11,6 @@
 module Remote.Git (
 	remote,
 	configRead,
-	repoAvail,
 	onLocalRepo,
 ) where
 
@@ -102,7 +101,7 @@ list autoinit = do
 			Nothing -> return r
 			Just url -> inRepo $ \g ->
 				Git.Construct.remoteNamed n $
-					Git.Construct.fromRemoteLocation (Git.fromConfigValue url) g
+					Git.Construct.fromRemoteLocation (Git.fromConfigValue url) False g
 
 {- Git remotes are normally set up using standard git commands, not
  - git-annex initremote and enableremote.
@@ -118,7 +117,7 @@ gitSetup :: SetupStage -> Maybe UUID -> Maybe CredPair -> RemoteConfig -> Remote
 gitSetup Init mu _ c _ = do
 	let location = maybe (giveup "Specify location=url") fromProposedAccepted $
 		M.lookup locationField c
-	r <- inRepo $ Git.Construct.fromRemoteLocation location
+	r <- inRepo $ Git.Construct.fromRemoteLocation location False
 	r' <- tryGitConfigRead False r False
 	let u = getUncachedUUID r'
 	if u == NoUUID
@@ -131,13 +130,18 @@ gitSetup (AutoEnable _) mu _ c _ = enableRemote mu c
 
 enableRemote :: Maybe UUID -> RemoteConfig -> Annex (RemoteConfig, UUID)
 enableRemote (Just u) c = do
-	inRepo $ Git.Command.run
-		[ Param "remote"
-		, Param "add"
-		, Param $ fromMaybe (giveup "no name") (SpecialRemote.lookupName c)
-		, Param $ maybe (giveup "no location") fromProposedAccepted (M.lookup locationField c)
-		]
+	rs <- Annex.getGitRemotes
+	unless (any (\r -> Git.remoteName r == Just cname) rs) $
+		inRepo $ Git.Command.run
+			[ Param "remote"
+			, Param "add"
+			, Param cname
+			, Param clocation
+			]
 	return (c, u)
+  where
+	cname = fromMaybe (giveup "no name") (SpecialRemote.lookupName c)
+	clocation = maybe (giveup "no location") fromProposedAccepted (M.lookup locationField c)
 enableRemote Nothing _ = giveup "unable to enable git remote with no specified uuid"
 
 {- It's assumed to be cheap to read the config of non-URL remotes, so this is
@@ -205,7 +209,7 @@ gen r u rc gc rs
 			, readonly = Git.repoIsHttp r
 			, appendonly = False
 			, untrustworthy = False
-			, availability = availabilityCalc r
+			, availability = repoAvail r
 			, remotetype = remote
 			, mkUnavailable = unavailable r u rc gc rs
 			, getInfo = gitRepoInfo new
@@ -227,22 +231,6 @@ unavailable r = gen r'
 			Nothing -> r { Git.location = Git.Unknown }
 		_ -> r -- already unavailable
 
-{- Checks relatively inexpensively if a repository is available for use. -}
-repoAvail :: Git.Repo -> Annex Bool
-repoAvail r 
-	| Git.repoIsHttp r = return True
-	| Git.GCrypt.isEncrypted r = do
-		g <- gitRepo
-		liftIO $ do
-			er <- Git.GCrypt.encryptedRemote g r
-			if Git.repoIsLocal er || Git.repoIsLocalUnknown er
-				then catchBoolIO $
-					void (Git.Config.read er) >> return True
-				else return True
-	| Git.repoIsUrl r = return True
-	| Git.repoIsLocalUnknown r = return False
-	| otherwise = liftIO $ isJust <$> catchMaybeIO (Git.Config.read r)
-
 {- Tries to read the config for a specified remote, updates state, and
  - returns the updated repo. -}
 tryGitConfigRead :: Bool -> Git.Repo -> Bool -> Annex Git.Repo
@@ -275,12 +263,12 @@ tryGitConfigRead autoinit r hasuuid
 		case v of
 			Right (r', val, _err) -> do
 				unless (isUUIDConfigured r' || S.null val || not mustincludeuuuid) $ do
-					warning $ "Failed to get annex.uuid configuration of repository " ++ Git.repoDescribe r
-					warning $ "Instead, got: " ++ show val
-					warning $ "This is unexpected; please check the network transport!"
+					warning $ UnquotedString $ "Failed to get annex.uuid configuration of repository " ++ Git.repoDescribe r
+					warning $ UnquotedString $ "Instead, got: " ++ show val
+					warning "This is unexpected; please check the network transport!"
 				return $ Right r'
 			Left l -> do
-				warning $ "Unable to parse git config from " ++ configloc
+				warning $ UnquotedString $ "Unable to parse git config from " ++ configloc
 				return $ Left (show l)
 
 	geturlconfig = Url.withUrlOptionsPromptingCreds $ \uo -> do
@@ -306,7 +294,7 @@ tryGitConfigRead autoinit r hasuuid
 				return r'
 			Left err -> do
 				set_ignore "not usable by git-annex" False
-				warning $ url ++ " " ++ err
+				warning $ UnquotedString $ url ++ " " ++ err
 				return r
 
 	{- Is this remote just not available, or does
@@ -323,9 +311,9 @@ tryGitConfigRead autoinit r hasuuid
 		case Git.remoteName r of
 			Nothing -> noop
 			Just n -> do
-				warning $ "Remote " ++ n ++ " " ++ msg ++ "; setting annex-ignore"
+				warning $ UnquotedString $ "Remote " ++ n ++ " " ++ msg ++ "; setting annex-ignore"
 				when longmessage $
-					warning $ "This could be a problem with the git-annex installation on the remote. Please make sure that git-annex-shell is available in PATH when you ssh into the remote. Once you have fixed the git-annex installation, run: git annex enableremote " ++ n
+					warning $ UnquotedString $ "This could be a problem with the git-annex installation on the remote. Please make sure that git-annex-shell is available in PATH when you ssh into the remote. Once you have fixed the git-annex installation, run: git annex enableremote " ++ n
 		setremote setRemoteIgnore True
 	
 	setremote setter v = case Git.remoteName r of
@@ -339,7 +327,7 @@ tryGitConfigRead autoinit r hasuuid
 		case Git.GCrypt.remoteRepoId g (Git.remoteName r) of
 			Nothing -> return r
 			Just v -> storeUpdatedRemote $ liftIO $ setUUID r $
-				genUUIDInNameSpace gCryptNameSpace v
+				genUUIDInNameSpace gCryptNameSpace (encodeBS v)
 
 	{- The local repo may not yet be initialized, so try to initialize
 	 - it if allowed. However, if that fails, still return the read
@@ -348,10 +336,11 @@ tryGitConfigRead autoinit r hasuuid
 		let check = do
 			Annex.BranchState.disableUpdate
 			catchNonAsync (autoInitialize (pure [])) $ \e ->
-				warning $ "Remote " ++ Git.repoDescribe r ++
+				warning $ UnquotedString $ "Remote " ++ Git.repoDescribe r ++
 					": "  ++ show e
 			Annex.getState Annex.repo
-		s <- newLocal r
+		let r' = r { Git.repoPathSpecifiedExplicitly = True }
+		s <- newLocal r'
 		liftIO $ Annex.eval s $ check
 			`finally` quiesce True
 		
@@ -359,7 +348,7 @@ tryGitConfigRead autoinit r hasuuid
 		unless hasuuid $ case Git.remoteName r of
 			Nothing -> noop
 			Just n -> do
-				warning $ "Remote " ++ n ++ " cannot currently be accessed."
+				warning $ UnquotedString $ "Remote " ++ n ++ " cannot currently be accessed."
 		return r
 		
 	configlistfields = if autoinit
@@ -495,16 +484,17 @@ copyFromRemote'' repo r st@(State connpool _ _ _ _) key file dest meterupdate vc
 	| not $ Git.repoIsUrl repo = guardUsable repo (giveup "cannot access remote") $ do
 		u <- getUUID
 		hardlink <- wantHardLink
-		let bwlimit = remoteAnnexBwLimit (gitconfig r)
+		let bwlimit = remoteAnnexBwLimitDownload (gitconfig r)
+			<|> remoteAnnexBwLimit (gitconfig r)
 		-- run copy from perspective of remote
 		onLocalFast st $ Annex.Content.prepSendAnnex' key >>= \case
-			Just (object, check) -> do
+			Just (object, _sz, check) -> do
 				let checksuccess = check >>= \case
 					Just err -> giveup err
 					Nothing -> return True
 				copier <- mkFileCopier hardlink st
 				(ok, v) <- runTransfer (Transfer Download u (fromKey id key))
-					file Nothing stdRetry $ \p ->
+					Nothing file Nothing stdRetry $ \p ->
 						metered (Just (combineMeterUpdate p meterupdate)) key bwlimit $ \_ p' -> 
 							copier object dest key p' checksuccess vc
 				if ok
@@ -556,25 +546,26 @@ copyToRemote' repo r st@(State connpool duc _ _ _) key file meterupdate
 	| otherwise = giveup "copying to non-ssh repo not supported"
   where
 	copylocal Nothing = giveup "content not available"
-	copylocal (Just (object, check)) = do
+	copylocal (Just (object, sz, check)) = do
 		-- The check action is going to be run in
 		-- the remote's Annex, but it needs access to the local
 		-- Annex monad's state.
 		checkio <- Annex.withCurrentState check
 		u <- getUUID
 		hardlink <- wantHardLink
-		let bwlimit = remoteAnnexBwLimit (gitconfig r)
+		let bwlimit = remoteAnnexBwLimitUpload (gitconfig r)
+			<|> remoteAnnexBwLimit (gitconfig r)
 		-- run copy from perspective of remote
 		res <- onLocalFast st $ ifM (Annex.Content.inAnnex key)
 			( return True
-			, runTransfer (Transfer Download u (fromKey id key)) file Nothing stdRetry $ \p -> do
+			, runTransfer (Transfer Download u (fromKey id key)) Nothing file Nothing stdRetry $ \p -> do
 				let verify = RemoteVerify r
 				copier <- mkFileCopier hardlink st
 				let rsp = RetrievalAllKeysSecure
 				let checksuccess = liftIO checkio >>= \case
 					Just err -> giveup err
 					Nothing -> return True
-				logStatusAfter key $ Annex.Content.getViaTmp rsp verify key file $ \dest ->
+				logStatusAfter key $ Annex.Content.getViaTmp rsp verify key file (Just sz) $ \dest ->
 					metered (Just (combineMeterUpdate meterupdate p)) key bwlimit $ \_ p' -> 
 						copier object (fromRawFilePath dest) key p' checksuccess verify
 			)
@@ -632,9 +623,11 @@ newLocal repo = do
 	(st, rd) <- liftIO $ Annex.new repo
 	debugenabled <- Annex.getRead Annex.debugenabled
 	debugselector <- Annex.getRead Annex.debugselector
+	force <- Annex.getRead Annex.force
 	return (st,  rd
 		{ Annex.debugenabled = debugenabled
 		, Annex.debugselector = debugselector
+		, Annex.force = force
 		})
 
 onLocal' :: LocalRemoteAnnex -> Annex a -> Annex a
@@ -770,7 +763,7 @@ mkState r u gc = do
 					let ok = u' == u
 					void $ liftIO $ tryPutMVar cv ok
 					unless ok $
-						warning $ Git.repoDescribe r ++ " is not the expected repository. The remote's annex-checkuuid configuration prevented noticing the change until now."
+						warning $ UnquotedString $ Git.repoDescribe r ++ " is not the expected repository. The remote's annex-checkuuid configuration prevented noticing the change until now."
 					return ok
 				, liftIO $ readMVar cv
 				)
